import keras
from keras.models import Sequential
from keras.layers import Dense, Dropout, Flatten
from keras.layers import Conv2D, MaxPooling2D, GlobalAveragePooling2D
from keras_preprocessing.image import ImageDataGenerator
from keras.applications.mobilenet_v2 import MobileNetV2, preprocess_input

batch_size = 16

train_datagen = ImageDataGenerator(
        preprocessing_function=preprocess_input,
        shear_range=0.2,
        zoom_range=0.2,
        horizontal_flip=True)

test_datagen = ImageDataGenerator(preprocessing_function=preprocess_input)

train_generator = train_datagen.flow_from_directory(
        'train',  # this is the target directory
        target_size=(224, 224),  # all images will be resized to 150x150
        batch_size=batch_size,
        class_mode='binary')

validation_generator = test_datagen.flow_from_directory(
        'val',  # this is the target directory
        target_size=(224, 224),  # all images will be resized to 150x150
        batch_size=batch_size,
        class_mode='binary')

test_generator = test_datagen.flow_from_directory(
        'test',  # this is the target directory
        target_size=(224, 224),  # all images will be resized to 150x150
        batch_size=batch_size,
        class_mode='binary')


pre_trained = MobileNetV2(input_shape=(224,224,3),  include_top = False)

print(pre_trained.summary())

for l in pre_trained.layers:
    l.trainable = False

model = Sequential()

model.add(pre_trained)
model.add(GlobalAveragePooling2D())
model.add(Dense(32, activation='relu'))
model.add(Dense(16, activation='relu'))
model.add(Dense(1, activation='sigmoid'))

print(model.summary())


callbacks = []
checkpoint = keras.callbacks.ModelCheckpoint(
    'transfer_{epoch:02d}-{val_loss:.8f}-{val_acc:.4f}.hdf5',
    verbose=1)
callbacks.append(checkpoint)

tb_callback = keras.callbacks.TensorBoard(
    log_dir='./logs')
callbacks.append(tb_callback)

optimizer = keras.optimizers.Adam(lr=0.0001)
model.compile(loss='binary_crossentropy',
              optimizer=optimizer,
              metrics=['accuracy'])

model.fit_generator(train_generator,
        epochs=10,
        callbacks=callbacks,
        validation_data=validation_generator)

model.evaluate_generator(test_generator)